# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,get_predicted)
S3method(as.data.frame,insight_residuals)
S3method(as.double,insight_aux)
S3method(as.matrix,get_predicted)
S3method(as.numeric,insight_aux)
S3method(clean_names,character)
S3method(clean_names,default)
S3method(clean_parameters,BFBayesFactor)
S3method(clean_parameters,MCMCglmm)
S3method(clean_parameters,afex_aov)
S3method(clean_parameters,aovlist)
S3method(clean_parameters,bamlss)
S3method(clean_parameters,blavaan)
S3method(clean_parameters,brmsfit)
S3method(clean_parameters,default)
S3method(clean_parameters,emmGrid)
S3method(clean_parameters,emm_list)
S3method(clean_parameters,glmm)
S3method(clean_parameters,lavaan)
S3method(clean_parameters,merModList)
S3method(clean_parameters,mlm)
S3method(clean_parameters,model_fit)
S3method(clean_parameters,stanfit)
S3method(clean_parameters,stanmvreg)
S3method(clean_parameters,stanreg)
S3method(clean_parameters,wbgee)
S3method(clean_parameters,wbm)
S3method(display,array)
S3method(display,data.frame)
S3method(display,matrix)
S3method(display,table)
S3method(ellipsis_info,ListLavaan)
S3method(ellipsis_info,ListModels)
S3method(ellipsis_info,ListObjects)
S3method(ellipsis_info,ListRegressions)
S3method(ellipsis_info,ListVarious)
S3method(ellipsis_info,default)
S3method(find_algorithm,Arima)
S3method(find_algorithm,BBmm)
S3method(find_algorithm,BBreg)
S3method(find_algorithm,CmdStanFit)
S3method(find_algorithm,Gam)
S3method(find_algorithm,LORgee)
S3method(find_algorithm,MixMod)
S3method(find_algorithm,afex_aov)
S3method(find_algorithm,bayesQR)
S3method(find_algorithm,bayesx)
S3method(find_algorithm,bigglm)
S3method(find_algorithm,biglm)
S3method(find_algorithm,blavaan)
S3method(find_algorithm,brmsfit)
S3method(find_algorithm,crq)
S3method(find_algorithm,default)
S3method(find_algorithm,flac)
S3method(find_algorithm,flic)
S3method(find_algorithm,gam)
S3method(find_algorithm,gamlss)
S3method(find_algorithm,glimML)
S3method(find_algorithm,glm)
S3method(find_algorithm,glmmTMB)
S3method(find_algorithm,glmrob)
S3method(find_algorithm,lm)
S3method(find_algorithm,lmRob)
S3method(find_algorithm,lme)
S3method(find_algorithm,lmrob)
S3method(find_algorithm,logistf)
S3method(find_algorithm,merMod)
S3method(find_algorithm,merModList)
S3method(find_algorithm,mixed)
S3method(find_algorithm,nestedLogit)
S3method(find_algorithm,rlmerMod)
S3method(find_algorithm,rq)
S3method(find_algorithm,rqs)
S3method(find_algorithm,rqss)
S3method(find_algorithm,scam)
S3method(find_algorithm,speedglm)
S3method(find_algorithm,speedlm)
S3method(find_algorithm,stanfit)
S3method(find_algorithm,stanreg)
S3method(find_algorithm,systemfit)
S3method(find_auxiliary,brmsfit)
S3method(find_auxiliary,default)
S3method(find_formula,BBmm)
S3method(find_formula,BFBayesFactor)
S3method(find_formula,BGGM)
S3method(find_formula,DirichletRegModel)
S3method(find_formula,HLfit)
S3method(find_formula,LORgee)
S3method(find_formula,MANOVA)
S3method(find_formula,MCMCglmm)
S3method(find_formula,MixMod)
S3method(find_formula,RM)
S3method(find_formula,SemiParBIV)
S3method(find_formula,afex_aov)
S3method(find_formula,aft)
S3method(find_formula,anova)
S3method(find_formula,aovlist)
S3method(find_formula,asym)
S3method(find_formula,averaging)
S3method(find_formula,bamlss)
S3method(find_formula,betamfx)
S3method(find_formula,betaor)
S3method(find_formula,betareg)
S3method(find_formula,bife)
S3method(find_formula,brmsfit)
S3method(find_formula,btergm)
S3method(find_formula,censReg)
S3method(find_formula,cgamm)
S3method(find_formula,cglm)
S3method(find_formula,clm)
S3method(find_formula,clm2)
S3method(find_formula,clmm)
S3method(find_formula,clmm2)
S3method(find_formula,coxme)
S3method(find_formula,cpglmm)
S3method(find_formula,data.frame)
S3method(find_formula,default)
S3method(find_formula,deltaMethod)
S3method(find_formula,estimate_contrasts)
S3method(find_formula,estimate_means)
S3method(find_formula,estimate_slopes)
S3method(find_formula,externVar)
S3method(find_formula,externX)
S3method(find_formula,feglm)
S3method(find_formula,feis)
S3method(find_formula,felm)
S3method(find_formula,fixest)
S3method(find_formula,gam)
S3method(find_formula,gamlss)
S3method(find_formula,gamm)
S3method(find_formula,gee)
S3method(find_formula,glht)
S3method(find_formula,glimML)
S3method(find_formula,glmgee)
S3method(find_formula,glmm)
S3method(find_formula,glmmPQL)
S3method(find_formula,glmmTMB)
S3method(find_formula,glmmadmb)
S3method(find_formula,gls)
S3method(find_formula,hglm)
S3method(find_formula,hurdle)
S3method(find_formula,ivFixed)
S3method(find_formula,iv_robust)
S3method(find_formula,ivprobit)
S3method(find_formula,ivreg)
S3method(find_formula,joint)
S3method(find_formula,lcmm)
S3method(find_formula,list)
S3method(find_formula,lme)
S3method(find_formula,logitmfx)
S3method(find_formula,logitor)
S3method(find_formula,logitr)
S3method(find_formula,lqmm)
S3method(find_formula,marginaleffects)
S3method(find_formula,maxLik)
S3method(find_formula,maxim)
S3method(find_formula,mcmc.list)
S3method(find_formula,mediate)
S3method(find_formula,merMod)
S3method(find_formula,merModList)
S3method(find_formula,meta_bma)
S3method(find_formula,meta_fixed)
S3method(find_formula,meta_random)
S3method(find_formula,metaplus)
S3method(find_formula,mhurdle)
S3method(find_formula,mira)
S3method(find_formula,mixed)
S3method(find_formula,mixor)
S3method(find_formula,mjoint)
S3method(find_formula,mmclogit)
S3method(find_formula,mmrm)
S3method(find_formula,mmrm_fit)
S3method(find_formula,mmrm_tmb)
S3method(find_formula,model_fit)
S3method(find_formula,mvord)
S3method(find_formula,negbinirr)
S3method(find_formula,negbinmfx)
S3method(find_formula,nestedLogit)
S3method(find_formula,nlmerMod)
S3method(find_formula,oohbchoice)
S3method(find_formula,pgmm)
S3method(find_formula,plm)
S3method(find_formula,poissonirr)
S3method(find_formula,poissonmfx)
S3method(find_formula,probitmfx)
S3method(find_formula,pstpm2)
S3method(find_formula,rlmerMod)
S3method(find_formula,rma)
S3method(find_formula,sdmTMB)
S3method(find_formula,selection)
S3method(find_formula,sem)
S3method(find_formula,stanmvreg)
S3method(find_formula,stanreg)
S3method(find_formula,stpm2)
S3method(find_formula,svy2lme)
S3method(find_formula,svy_vglm)
S3method(find_formula,systemfit)
S3method(find_formula,tobit)
S3method(find_formula,wbgee)
S3method(find_formula,wbm)
S3method(find_formula,zcpglm)
S3method(find_formula,zeroinfl)
S3method(find_formula,zerotrunc)
S3method(find_parameters,BBmm)
S3method(find_parameters,BBreg)
S3method(find_parameters,BFBayesFactor)
S3method(find_parameters,BGGM)
S3method(find_parameters,CmdStanFit)
S3method(find_parameters,DirichletRegModel)
S3method(find_parameters,Gam)
S3method(find_parameters,HLfit)
S3method(find_parameters,MCMCglmm)
S3method(find_parameters,MixMod)
S3method(find_parameters,Rchoice)
S3method(find_parameters,SemiParBIV)
S3method(find_parameters,aareg)
S3method(find_parameters,afex_aov)
S3method(find_parameters,anova.rms)
S3method(find_parameters,aovlist)
S3method(find_parameters,asym)
S3method(find_parameters,averaging)
S3method(find_parameters,bamlss)
S3method(find_parameters,bayesQR)
S3method(find_parameters,bayesx)
S3method(find_parameters,bcplm)
S3method(find_parameters,betamfx)
S3method(find_parameters,betaor)
S3method(find_parameters,betareg)
S3method(find_parameters,bfsl)
S3method(find_parameters,blavaan)
S3method(find_parameters,bracl)
S3method(find_parameters,brmsfit)
S3method(find_parameters,brmultinom)
S3method(find_parameters,btergm)
S3method(find_parameters,cgam)
S3method(find_parameters,clm2)
S3method(find_parameters,clmm2)
S3method(find_parameters,coxme)
S3method(find_parameters,coxph)
S3method(find_parameters,cpglmm)
S3method(find_parameters,crq)
S3method(find_parameters,crqs)
S3method(find_parameters,crr)
S3method(find_parameters,data.frame)
S3method(find_parameters,default)
S3method(find_parameters,deltaMethod)
S3method(find_parameters,emmGrid)
S3method(find_parameters,emm_list)
S3method(find_parameters,estimate_contrasts)
S3method(find_parameters,estimate_means)
S3method(find_parameters,estimate_slopes)
S3method(find_parameters,externVar)
S3method(find_parameters,externX)
S3method(find_parameters,flexsurvreg)
S3method(find_parameters,gam)
S3method(find_parameters,gamlss)
S3method(find_parameters,gamm)
S3method(find_parameters,gbm)
S3method(find_parameters,glht)
S3method(find_parameters,glimML)
S3method(find_parameters,glmgee)
S3method(find_parameters,glmm)
S3method(find_parameters,glmmTMB)
S3method(find_parameters,glmmadmb)
S3method(find_parameters,glmx)
S3method(find_parameters,hglm)
S3method(find_parameters,hurdle)
S3method(find_parameters,ivFixed)
S3method(find_parameters,ivprobit)
S3method(find_parameters,lavaan)
S3method(find_parameters,lcmm)
S3method(find_parameters,lme)
S3method(find_parameters,lmodel2)
S3method(find_parameters,logitmfx)
S3method(find_parameters,logitor)
S3method(find_parameters,lqm)
S3method(find_parameters,lqmm)
S3method(find_parameters,lrm)
S3method(find_parameters,manova)
S3method(find_parameters,maov)
S3method(find_parameters,marginaleffects)
S3method(find_parameters,marginaleffects.summary)
S3method(find_parameters,mcmc)
S3method(find_parameters,mcmc.list)
S3method(find_parameters,mediate)
S3method(find_parameters,merMod)
S3method(find_parameters,merModList)
S3method(find_parameters,meta_bma)
S3method(find_parameters,meta_fixed)
S3method(find_parameters,meta_random)
S3method(find_parameters,metaplus)
S3method(find_parameters,mhurdle)
S3method(find_parameters,mipo)
S3method(find_parameters,mira)
S3method(find_parameters,mixed)
S3method(find_parameters,mixor)
S3method(find_parameters,mjoint)
S3method(find_parameters,mle)
S3method(find_parameters,mle2)
S3method(find_parameters,mlm)
S3method(find_parameters,model_fit)
S3method(find_parameters,multinom)
S3method(find_parameters,multinom_weightit)
S3method(find_parameters,mvord)
S3method(find_parameters,negbinirr)
S3method(find_parameters,negbinmfx)
S3method(find_parameters,nestedLogit)
S3method(find_parameters,nlmerMod)
S3method(find_parameters,nls)
S3method(find_parameters,oohbchoice)
S3method(find_parameters,pgmm)
S3method(find_parameters,poissonirr)
S3method(find_parameters,poissonmfx)
S3method(find_parameters,polr)
S3method(find_parameters,probitmfx)
S3method(find_parameters,ridgelm)
S3method(find_parameters,riskRegression)
S3method(find_parameters,rlmerMod)
S3method(find_parameters,rma)
S3method(find_parameters,rms)
S3method(find_parameters,rqs)
S3method(find_parameters,rqss)
S3method(find_parameters,scam)
S3method(find_parameters,sdmTMB)
S3method(find_parameters,selection)
S3method(find_parameters,sem)
S3method(find_parameters,sim)
S3method(find_parameters,sim.merMod)
S3method(find_parameters,stanfit)
S3method(find_parameters,stanmvreg)
S3method(find_parameters,stanreg)
S3method(find_parameters,summary.lm)
S3method(find_parameters,survreg)
S3method(find_parameters,svy2lme)
S3method(find_parameters,svyolr)
S3method(find_parameters,systemfit)
S3method(find_parameters,tobit)
S3method(find_parameters,vgam)
S3method(find_parameters,wbgee)
S3method(find_parameters,wbm)
S3method(find_parameters,zcpglm)
S3method(find_parameters,zeroinfl)
S3method(find_parameters,zerotrunc)
S3method(find_predictors,afex_aov)
S3method(find_predictors,bfsl)
S3method(find_predictors,brmsfit)
S3method(find_predictors,default)
S3method(find_predictors,externVar)
S3method(find_predictors,externX)
S3method(find_predictors,fixest)
S3method(find_predictors,insight_formula)
S3method(find_predictors,lcmm)
S3method(find_predictors,logitr)
S3method(find_predictors,sdmTMB)
S3method(find_predictors,selection)
S3method(find_predictors,survey.design)
S3method(find_predictors,survey.design2)
S3method(find_predictors,svycoxph)
S3method(find_predictors,svyglm)
S3method(find_predictors,svyolr)
S3method(find_predictors,svysurvreg)
S3method(find_predictors,workflow)
S3method(find_random,afex_aov)
S3method(find_random,default)
S3method(find_random,insight_formula)
S3method(find_response,bfsl)
S3method(find_response,brmsfit)
S3method(find_response,default)
S3method(find_response,insight_formula)
S3method(find_response,joint)
S3method(find_response,logitr)
S3method(find_response,mediate)
S3method(find_response,mjoint)
S3method(find_response,model_fit)
S3method(find_response,selection)
S3method(find_response,tune_results)
S3method(find_response,workflow)
S3method(find_statistic,default)
S3method(find_statistic,estimate_contrasts)
S3method(find_statistic,estimate_means)
S3method(find_statistic,estimate_slopes)
S3method(find_terms,afex_aov)
S3method(find_terms,aovlist)
S3method(find_terms,bfsl)
S3method(find_terms,default)
S3method(find_terms,insight_formula)
S3method(find_terms,mipo)
S3method(find_transformation,character)
S3method(find_transformation,default)
S3method(find_weights,brmsfit)
S3method(find_weights,default)
S3method(find_weights,gls)
S3method(find_weights,lme)
S3method(find_weights,model_fit)
S3method(find_weights,survey.design)
S3method(find_weights,survey.design2)
S3method(find_weights,svycoxph)
S3method(find_weights,svyglm)
S3method(find_weights,svyolr)
S3method(find_weights,svysurvreg)
S3method(format,insight_formula)
S3method(format,n_grouplevels)
S3method(format_ci,bayestestR_ci)
S3method(format_ci,character)
S3method(format_ci,data.frame)
S3method(format_ci,factor)
S3method(format_ci,logical)
S3method(format_ci,numeric)
S3method(format_string,character)
S3method(format_string,data.frame)
S3method(format_string,default)
S3method(format_value,character)
S3method(format_value,data.frame)
S3method(format_value,double)
S3method(format_value,factor)
S3method(format_value,logical)
S3method(format_value,numeric)
S3method(get_call,default)
S3method(get_call,estimate_contrasts)
S3method(get_call,estimate_means)
S3method(get_call,estimate_slopes)
S3method(get_call,glm)
S3method(get_call,lm)
S3method(get_call,lmerMod)
S3method(get_call,merMod)
S3method(get_call,model_fit)
S3method(get_call,mvord)
S3method(get_call,stanreg)
S3method(get_data,Arima)
S3method(get_data,BBmm)
S3method(get_data,BFBayesFactor)
S3method(get_data,BGGM)
S3method(get_data,DirichletRegModel)
S3method(get_data,HLfit)
S3method(get_data,LORgee)
S3method(get_data,MANOVA)
S3method(get_data,MCMCglmm)
S3method(get_data,MixMod)
S3method(get_data,RM)
S3method(get_data,aareg)
S3method(get_data,afex_aov)
S3method(get_data,aft)
S3method(get_data,averaging)
S3method(get_data,betamfx)
S3method(get_data,betaor)
S3method(get_data,bfsl)
S3method(get_data,bife)
S3method(get_data,bigglm)
S3method(get_data,biglm)
S3method(get_data,blavaan)
S3method(get_data,bracl)
S3method(get_data,brmsfit)
S3method(get_data,censReg)
S3method(get_data,clm2)
S3method(get_data,clmm)
S3method(get_data,clmm2)
S3method(get_data,complmrob)
S3method(get_data,coxme)
S3method(get_data,coxph)
S3method(get_data,cpglmm)
S3method(get_data,data.frame)
S3method(get_data,default)
S3method(get_data,feglm)
S3method(get_data,feis)
S3method(get_data,felm)
S3method(get_data,fixest)
S3method(get_data,gam)
S3method(get_data,gamlss)
S3method(get_data,gamm)
S3method(get_data,gbm)
S3method(get_data,gee)
S3method(get_data,geeglm)
S3method(get_data,glht)
S3method(get_data,glimML)
S3method(get_data,glmgee)
S3method(get_data,glmm)
S3method(get_data,glmmTMB)
S3method(get_data,glmmadmb)
S3method(get_data,gls)
S3method(get_data,gmnl)
S3method(get_data,gnls)
S3method(get_data,htest)
S3method(get_data,hurdle)
S3method(get_data,ivFixed)
S3method(get_data,iv_robust)
S3method(get_data,ivprobit)
S3method(get_data,ivreg)
S3method(get_data,lavaan)
S3method(get_data,list)
S3method(get_data,lme)
S3method(get_data,logitmfx)
S3method(get_data,logitor)
S3method(get_data,lqmm)
S3method(get_data,maxLik)
S3method(get_data,maxim)
S3method(get_data,mcmc.list)
S3method(get_data,mediate)
S3method(get_data,merMod)
S3method(get_data,merModList)
S3method(get_data,meta_bma)
S3method(get_data,meta_fixed)
S3method(get_data,meta_random)
S3method(get_data,metaplus)
S3method(get_data,mhurdle)
S3method(get_data,mipo)
S3method(get_data,mira)
S3method(get_data,mixed)
S3method(get_data,mixor)
S3method(get_data,mjoint)
S3method(get_data,mle)
S3method(get_data,mle2)
S3method(get_data,mlogit)
S3method(get_data,mmrm)
S3method(get_data,mmrm_fit)
S3method(get_data,mmrm_tmb)
S3method(get_data,model_fit)
S3method(get_data,negbinirr)
S3method(get_data,negbinmfx)
S3method(get_data,nestedLogit)
S3method(get_data,nlrq)
S3method(get_data,nls)
S3method(get_data,pgmm)
S3method(get_data,phyloglm)
S3method(get_data,phylolm)
S3method(get_data,plm)
S3method(get_data,poissonirr)
S3method(get_data,poissonmfx)
S3method(get_data,probitmfx)
S3method(get_data,pstpm2)
S3method(get_data,rlmerMod)
S3method(get_data,rma)
S3method(get_data,robmixglm)
S3method(get_data,rqss)
S3method(get_data,selection)
S3method(get_data,sem)
S3method(get_data,stanmvreg)
S3method(get_data,stanreg)
S3method(get_data,stpm2)
S3method(get_data,summary.lm)
S3method(get_data,survey.design)
S3method(get_data,survey.design2)
S3method(get_data,survfit)
S3method(get_data,svy_vglm)
S3method(get_data,svycoxph)
S3method(get_data,svyglm)
S3method(get_data,svyolr)
S3method(get_data,svysurvreg)
S3method(get_data,tobit)
S3method(get_data,vgam)
S3method(get_data,vglm)
S3method(get_data,wbgee)
S3method(get_data,wbm)
S3method(get_data,workflow)
S3method(get_data,zcpglm)
S3method(get_data,zeroinfl)
S3method(get_data,zerotrunc)
S3method(get_datagrid,character)
S3method(get_datagrid,comparisons)
S3method(get_datagrid,data.frame)
S3method(get_datagrid,datagrid)
S3method(get_datagrid,default)
S3method(get_datagrid,double)
S3method(get_datagrid,emmGrid)
S3method(get_datagrid,emm_list)
S3method(get_datagrid,factor)
S3method(get_datagrid,logical)
S3method(get_datagrid,logitr)
S3method(get_datagrid,numeric)
S3method(get_datagrid,predictions)
S3method(get_datagrid,slopes)
S3method(get_datagrid,visualisation_matrix)
S3method(get_datagrid,wbm)
S3method(get_deviance,MixMod)
S3method(get_deviance,default)
S3method(get_deviance,glmerMod)
S3method(get_deviance,glmmTMB)
S3method(get_deviance,lmerMod)
S3method(get_deviance,lrm)
S3method(get_deviance,model_fit)
S3method(get_deviance,stanreg)
S3method(get_df,afex_aov)
S3method(get_df,aovlist)
S3method(get_df,betamfx)
S3method(get_df,betaor)
S3method(get_df,coeftest)
S3method(get_df,complmrob)
S3method(get_df,default)
S3method(get_df,emmGrid)
S3method(get_df,emm_list)
S3method(get_df,estimate_contrasts)
S3method(get_df,estimate_means)
S3method(get_df,estimate_slopes)
S3method(get_df,externVar)
S3method(get_df,externX)
S3method(get_df,fixest)
S3method(get_df,fixest_multi)
S3method(get_df,garch)
S3method(get_df,glht)
S3method(get_df,glmmTMB)
S3method(get_df,lcmm)
S3method(get_df,lme)
S3method(get_df,lmerMod)
S3method(get_df,lmerModTest)
S3method(get_df,logitmfx)
S3method(get_df,logitor)
S3method(get_df,mediate)
S3method(get_df,mhurdle)
S3method(get_df,mipo)
S3method(get_df,mira)
S3method(get_df,mmrm)
S3method(get_df,mmrm_fit)
S3method(get_df,mmrm_tmb)
S3method(get_df,model_fit)
S3method(get_df,multinom)
S3method(get_df,negbinirr)
S3method(get_df,negbinmfx)
S3method(get_df,nestedLogit)
S3method(get_df,nlrq)
S3method(get_df,phyloglm)
S3method(get_df,phylolm)
S3method(get_df,plm)
S3method(get_df,poissonirr)
S3method(get_df,poissonmfx)
S3method(get_df,probitmfx)
S3method(get_df,rlm)
S3method(get_df,selection)
S3method(get_df,serp)
S3method(get_df,svy2lme)
S3method(get_df,systemfit)
S3method(get_df,truncreg)
S3method(get_df,vgam)
S3method(get_dispersion,brmsfit)
S3method(get_dispersion,default)
S3method(get_dispersion,glm)
S3method(get_dispersion,glmerMod)
S3method(get_dispersion,glmmTMB)
S3method(get_dispersion,model_fit)
S3method(get_family,default)
S3method(get_family,list)
S3method(get_family,model_fit)
S3method(get_family,sdmTMB)
S3method(get_intercept,default)
S3method(get_intercept,stanreg)
S3method(get_loglikelihood,afex_aov)
S3method(get_loglikelihood,cpglm)
S3method(get_loglikelihood,crr)
S3method(get_loglikelihood,default)
S3method(get_loglikelihood,externVar)
S3method(get_loglikelihood,externX)
S3method(get_loglikelihood,gam)
S3method(get_loglikelihood,gamm)
S3method(get_loglikelihood,glm)
S3method(get_loglikelihood,glmerMod)
S3method(get_loglikelihood,glmmTMB)
S3method(get_loglikelihood,hglm)
S3method(get_loglikelihood,iv_robust)
S3method(get_loglikelihood,ivreg)
S3method(get_loglikelihood,lcmm)
S3method(get_loglikelihood,list)
S3method(get_loglikelihood,lm)
S3method(get_loglikelihood,lm_robust)
S3method(get_loglikelihood,lmerMod)
S3method(get_loglikelihood,mblogit)
S3method(get_loglikelihood,mclogit)
S3method(get_loglikelihood,mlogit)
S3method(get_loglikelihood,model_fit)
S3method(get_loglikelihood,phyloglm)
S3method(get_loglikelihood,phylolm)
S3method(get_loglikelihood,plm)
S3method(get_loglikelihood,stanreg)
S3method(get_loglikelihood,svycoxph)
S3method(get_mixed_info,MixMod)
S3method(get_mixed_info,brmsfit)
S3method(get_mixed_info,clmm)
S3method(get_mixed_info,coxme)
S3method(get_mixed_info,cpglmm)
S3method(get_mixed_info,default)
S3method(get_mixed_info,glmmTMB)
S3method(get_mixed_info,glmmadmb)
S3method(get_mixed_info,lme)
S3method(get_mixed_info,mjoint)
S3method(get_mixed_info,stanreg)
S3method(get_modelmatrix,BFBayesFactor)
S3method(get_modelmatrix,afex_aov)
S3method(get_modelmatrix,betareg)
S3method(get_modelmatrix,bracl)
S3method(get_modelmatrix,brmsfit)
S3method(get_modelmatrix,clmm)
S3method(get_modelmatrix,coxme)
S3method(get_modelmatrix,cpglmm)
S3method(get_modelmatrix,default)
S3method(get_modelmatrix,gls)
S3method(get_modelmatrix,iv_robust)
S3method(get_modelmatrix,ivreg)
S3method(get_modelmatrix,lm_robust)
S3method(get_modelmatrix,lme)
S3method(get_modelmatrix,merMod)
S3method(get_modelmatrix,rlm)
S3method(get_modelmatrix,serp)
S3method(get_modelmatrix,svycoxph)
S3method(get_modelmatrix,svyglm)
S3method(get_modelmatrix,svyolr)
S3method(get_modelmatrix,svysurvreg)
S3method(get_parameters,BBmm)
S3method(get_parameters,BBreg)
S3method(get_parameters,BFBayesFactor)
S3method(get_parameters,BGGM)
S3method(get_parameters,CmdStanFit)
S3method(get_parameters,DirichletRegModel)
S3method(get_parameters,Gam)
S3method(get_parameters,HLfit)
S3method(get_parameters,MCMCglmm)
S3method(get_parameters,MixMod)
S3method(get_parameters,Rchoice)
S3method(get_parameters,SemiParBIV)
S3method(get_parameters,aareg)
S3method(get_parameters,afex_aov)
S3method(get_parameters,aov)
S3method(get_parameters,aovlist)
S3method(get_parameters,asym)
S3method(get_parameters,averaging)
S3method(get_parameters,bamlss)
S3method(get_parameters,bayesQR)
S3method(get_parameters,bayesx)
S3method(get_parameters,bcplm)
S3method(get_parameters,betamfx)
S3method(get_parameters,betaor)
S3method(get_parameters,betareg)
S3method(get_parameters,bfsl)
S3method(get_parameters,blavaan)
S3method(get_parameters,blrm)
S3method(get_parameters,bracl)
S3method(get_parameters,brmsfit)
S3method(get_parameters,brmultinom)
S3method(get_parameters,btergm)
S3method(get_parameters,cgam)
S3method(get_parameters,clm2)
S3method(get_parameters,clmm2)
S3method(get_parameters,coxme)
S3method(get_parameters,coxph)
S3method(get_parameters,cpglmm)
S3method(get_parameters,crq)
S3method(get_parameters,crqs)
S3method(get_parameters,crr)
S3method(get_parameters,data.frame)
S3method(get_parameters,default)
S3method(get_parameters,deltaMethod)
S3method(get_parameters,emmGrid)
S3method(get_parameters,emm_list)
S3method(get_parameters,epi.2by2)
S3method(get_parameters,estimate_contrasts)
S3method(get_parameters,estimate_means)
S3method(get_parameters,estimate_slopes)
S3method(get_parameters,externVar)
S3method(get_parameters,externX)
S3method(get_parameters,flexsurvreg)
S3method(get_parameters,gam)
S3method(get_parameters,gamlss)
S3method(get_parameters,gamm)
S3method(get_parameters,gbm)
S3method(get_parameters,ggcomparisons)
S3method(get_parameters,glht)
S3method(get_parameters,glimML)
S3method(get_parameters,glmgee)
S3method(get_parameters,glmm)
S3method(get_parameters,glmmTMB)
S3method(get_parameters,glmmadmb)
S3method(get_parameters,glmx)
S3method(get_parameters,hglm)
S3method(get_parameters,htest)
S3method(get_parameters,hurdle)
S3method(get_parameters,ivFixed)
S3method(get_parameters,ivprobit)
S3method(get_parameters,lavaan)
S3method(get_parameters,lcmm)
S3method(get_parameters,lm_robust)
S3method(get_parameters,lme)
S3method(get_parameters,lmodel2)
S3method(get_parameters,logitmfx)
S3method(get_parameters,logitor)
S3method(get_parameters,lqm)
S3method(get_parameters,lqmm)
S3method(get_parameters,lrm)
S3method(get_parameters,manova)
S3method(get_parameters,maov)
S3method(get_parameters,marginaleffects)
S3method(get_parameters,marginaleffects.summary)
S3method(get_parameters,margins)
S3method(get_parameters,mblogit)
S3method(get_parameters,mclogit)
S3method(get_parameters,mcmc)
S3method(get_parameters,mcmc.list)
S3method(get_parameters,mediate)
S3method(get_parameters,merMod)
S3method(get_parameters,merModList)
S3method(get_parameters,meta_bma)
S3method(get_parameters,meta_fixed)
S3method(get_parameters,meta_random)
S3method(get_parameters,metaplus)
S3method(get_parameters,mhurdle)
S3method(get_parameters,mipo)
S3method(get_parameters,mira)
S3method(get_parameters,mixed)
S3method(get_parameters,mixor)
S3method(get_parameters,mjoint)
S3method(get_parameters,mle)
S3method(get_parameters,mle2)
S3method(get_parameters,mlm)
S3method(get_parameters,model_fit)
S3method(get_parameters,multinom)
S3method(get_parameters,multinom_weightit)
S3method(get_parameters,mvord)
S3method(get_parameters,negbinirr)
S3method(get_parameters,negbinmfx)
S3method(get_parameters,nestedLogit)
S3method(get_parameters,nlmerMod)
S3method(get_parameters,oohbchoice)
S3method(get_parameters,ordinal_weightit)
S3method(get_parameters,orm)
S3method(get_parameters,pgmm)
S3method(get_parameters,poissonirr)
S3method(get_parameters,poissonmfx)
S3method(get_parameters,polr)
S3method(get_parameters,probitmfx)
S3method(get_parameters,ridgelm)
S3method(get_parameters,riskRegression)
S3method(get_parameters,rlmerMod)
S3method(get_parameters,rma)
S3method(get_parameters,rms)
S3method(get_parameters,rqs)
S3method(get_parameters,rqss)
S3method(get_parameters,scam)
S3method(get_parameters,sdmTMB)
S3method(get_parameters,selection)
S3method(get_parameters,sem)
S3method(get_parameters,sim)
S3method(get_parameters,sim.merMod)
S3method(get_parameters,stanfit)
S3method(get_parameters,stanmvreg)
S3method(get_parameters,stanreg)
S3method(get_parameters,summary.lm)
S3method(get_parameters,survreg)
S3method(get_parameters,svy2lme)
S3method(get_parameters,svyolr)
S3method(get_parameters,systemfit)
S3method(get_parameters,tobit)
S3method(get_parameters,vgam)
S3method(get_parameters,wbgee)
S3method(get_parameters,wbm)
S3method(get_parameters,zcpglm)
S3method(get_parameters,zeroinfl)
S3method(get_parameters,zerotrunc)
S3method(get_predicted,Gam)
S3method(get_predicted,MixMod)
S3method(get_predicted,afex_aov)
S3method(get_predicted,bife)
S3method(get_predicted,bracl)
S3method(get_predicted,brmsfit)
S3method(get_predicted,clm)
S3method(get_predicted,coxme)
S3method(get_predicted,coxph)
S3method(get_predicted,crr)
S3method(get_predicted,data.frame)
S3method(get_predicted,default)
S3method(get_predicted,fa)
S3method(get_predicted,faMain)
S3method(get_predicted,fixest)
S3method(get_predicted,gam)
S3method(get_predicted,gamlss)
S3method(get_predicted,gamm)
S3method(get_predicted,glm)
S3method(get_predicted,glmgee)
S3method(get_predicted,glmmTMB)
S3method(get_predicted,hglm)
S3method(get_predicted,htest)
S3method(get_predicted,hurdle)
S3method(get_predicted,list)
S3method(get_predicted,lm)
S3method(get_predicted,lmerMod)
S3method(get_predicted,lrm)
S3method(get_predicted,merMod)
S3method(get_predicted,multinom)
S3method(get_predicted,nestedLogit)
S3method(get_predicted,phylolm)
S3method(get_predicted,polr)
S3method(get_predicted,prcomp)
S3method(get_predicted,principal)
S3method(get_predicted,rlm)
S3method(get_predicted,rma)
S3method(get_predicted,sdmTMB)
S3method(get_predicted,stanreg)
S3method(get_predicted,survreg)
S3method(get_predicted,zeroinfl)
S3method(get_predicted_ci,bracl)
S3method(get_predicted_ci,default)
S3method(get_predicted_ci,mlm)
S3method(get_predicted_ci,multinom)
S3method(get_predicted_ci,polr)
S3method(get_priors,BFBayesFactor)
S3method(get_priors,bcplm)
S3method(get_priors,blavaan)
S3method(get_priors,brmsfit)
S3method(get_priors,mcmc.list)
S3method(get_priors,meta_fixed)
S3method(get_priors,meta_random)
S3method(get_priors,stanmvreg)
S3method(get_priors,stanreg)
S3method(get_random,afex_aov)
S3method(get_random,default)
S3method(get_residuals,afex_aov)
S3method(get_residuals,brmsfit)
S3method(get_residuals,coxph)
S3method(get_residuals,crr)
S3method(get_residuals,default)
S3method(get_residuals,fa)
S3method(get_residuals,hglm)
S3method(get_residuals,model_fit)
S3method(get_residuals,omega)
S3method(get_residuals,parameters_efa)
S3method(get_residuals,principal)
S3method(get_residuals,psych)
S3method(get_residuals,slm)
S3method(get_residuals,vgam)
S3method(get_residuals,vglm)
S3method(get_response,DirichletRegModel)
S3method(get_response,bfsl)
S3method(get_response,default)
S3method(get_response,nestedLogit)
S3method(get_simulated,Gam)
S3method(get_simulated,MixMod)
S3method(get_simulated,afex_aov)
S3method(get_simulated,betareg)
S3method(get_simulated,bife)
S3method(get_simulated,bracl)
S3method(get_simulated,brmsfit)
S3method(get_simulated,clm)
S3method(get_simulated,coxme)
S3method(get_simulated,coxph)
S3method(get_simulated,crr)
S3method(get_simulated,data.frame)
S3method(get_simulated,default)
S3method(get_simulated,fa)
S3method(get_simulated,faMain)
S3method(get_simulated,fixest)
S3method(get_simulated,gam)
S3method(get_simulated,gamlss)
S3method(get_simulated,gamm)
S3method(get_simulated,glm)
S3method(get_simulated,glmgee)
S3method(get_simulated,glmmTMB)
S3method(get_simulated,hglm)
S3method(get_simulated,htest)
S3method(get_simulated,hurdle)
S3method(get_simulated,list)
S3method(get_simulated,lm)
S3method(get_simulated,lmerMod)
S3method(get_simulated,lrm)
S3method(get_simulated,merMod)
S3method(get_simulated,multinom)
S3method(get_simulated,phylolm)
S3method(get_simulated,polr)
S3method(get_simulated,prcomp)
S3method(get_simulated,principal)
S3method(get_simulated,rlm)
S3method(get_simulated,rma)
S3method(get_simulated,sdmTMB)
S3method(get_simulated,stanreg)
S3method(get_simulated,survreg)
S3method(get_simulated,zeroinfl)
S3method(get_statistic,Arima)
S3method(get_statistic,BBmm)
S3method(get_statistic,DirichletRegModel)
S3method(get_statistic,Gam)
S3method(get_statistic,HLfit)
S3method(get_statistic,LORgee)
S3method(get_statistic,MANOVA)
S3method(get_statistic,MixMod)
S3method(get_statistic,RM)
S3method(get_statistic,Rchoice)
S3method(get_statistic,Sarlm)
S3method(get_statistic,SemiParBIV)
S3method(get_statistic,aareg)
S3method(get_statistic,afex_aov)
S3method(get_statistic,anova.rms)
S3method(get_statistic,aov)
S3method(get_statistic,asym)
S3method(get_statistic,averaging)
S3method(get_statistic,bayesx)
S3method(get_statistic,betamfx)
S3method(get_statistic,betaor)
S3method(get_statistic,betareg)
S3method(get_statistic,bfsl)
S3method(get_statistic,bife)
S3method(get_statistic,bigglm)
S3method(get_statistic,biglm)
S3method(get_statistic,bracl)
S3method(get_statistic,brmultinom)
S3method(get_statistic,btergm)
S3method(get_statistic,censReg)
S3method(get_statistic,cgam)
S3method(get_statistic,clm2)
S3method(get_statistic,clmm2)
S3method(get_statistic,coeftest)
S3method(get_statistic,complmrob)
S3method(get_statistic,coxme)
S3method(get_statistic,coxph)
S3method(get_statistic,coxr)
S3method(get_statistic,cpglm)
S3method(get_statistic,cpglmm)
S3method(get_statistic,crch)
S3method(get_statistic,crq)
S3method(get_statistic,crqs)
S3method(get_statistic,crr)
S3method(get_statistic,default)
S3method(get_statistic,deltaMethod)
S3method(get_statistic,emmGrid)
S3method(get_statistic,emm_list)
S3method(get_statistic,epi.2by2)
S3method(get_statistic,ergm)
S3method(get_statistic,estimate_contrasts)
S3method(get_statistic,estimate_means)
S3method(get_statistic,estimate_slopes)
S3method(get_statistic,externVar)
S3method(get_statistic,externX)
S3method(get_statistic,feis)
S3method(get_statistic,fixest)
S3method(get_statistic,flac)
S3method(get_statistic,flexsurvreg)
S3method(get_statistic,flic)
S3method(get_statistic,gam)
S3method(get_statistic,gamlss)
S3method(get_statistic,gamm)
S3method(get_statistic,garch)
S3method(get_statistic,gee)
S3method(get_statistic,geeglm)
S3method(get_statistic,ggcomparisons)
S3method(get_statistic,glht)
S3method(get_statistic,glimML)
S3method(get_statistic,glmgee)
S3method(get_statistic,glmm)
S3method(get_statistic,glmmTMB)
S3method(get_statistic,glmmadmb)
S3method(get_statistic,glmx)
S3method(get_statistic,hglm)
S3method(get_statistic,htest)
S3method(get_statistic,hurdle)
S3method(get_statistic,ivFixed)
S3method(get_statistic,ivprobit)
S3method(get_statistic,lavaan)
S3method(get_statistic,lcmm)
S3method(get_statistic,list)
S3method(get_statistic,lm_robust)
S3method(get_statistic,lme)
S3method(get_statistic,lmerModLmerTest)
S3method(get_statistic,lmodel2)
S3method(get_statistic,logistf)
S3method(get_statistic,logitmfx)
S3method(get_statistic,logitor)
S3method(get_statistic,lqm)
S3method(get_statistic,lqmm)
S3method(get_statistic,lrm)
S3method(get_statistic,manova)
S3method(get_statistic,maov)
S3method(get_statistic,marginaleffects)
S3method(get_statistic,marginaleffects.summary)
S3method(get_statistic,margins)
S3method(get_statistic,maxLik)
S3method(get_statistic,mblogit)
S3method(get_statistic,mclogit)
S3method(get_statistic,mediate)
S3method(get_statistic,merModList)
S3method(get_statistic,metaplus)
S3method(get_statistic,mhurdle)
S3method(get_statistic,mipo)
S3method(get_statistic,mira)
S3method(get_statistic,mixor)
S3method(get_statistic,mjoint)
S3method(get_statistic,mle)
S3method(get_statistic,mle2)
S3method(get_statistic,mlm)
S3method(get_statistic,mlogit)
S3method(get_statistic,mmrm)
S3method(get_statistic,mmrm_fit)
S3method(get_statistic,mmrm_tmb)
S3method(get_statistic,model_fit)
S3method(get_statistic,multinom)
S3method(get_statistic,multinom_weightit)
S3method(get_statistic,mvord)
S3method(get_statistic,negbin)
S3method(get_statistic,negbinirr)
S3method(get_statistic,negbinmfx)
S3method(get_statistic,nestedLogit)
S3method(get_statistic,nlrq)
S3method(get_statistic,ols)
S3method(get_statistic,oohbchoice)
S3method(get_statistic,ordinal_weightit)
S3method(get_statistic,orm)
S3method(get_statistic,pgmm)
S3method(get_statistic,plm)
S3method(get_statistic,poissonirr)
S3method(get_statistic,poissonmfx)
S3method(get_statistic,polr)
S3method(get_statistic,probitmfx)
S3method(get_statistic,psm)
S3method(get_statistic,ridgelm)
S3method(get_statistic,riskRegression)
S3method(get_statistic,rma)
S3method(get_statistic,rms)
S3method(get_statistic,robmixglm)
S3method(get_statistic,rq)
S3method(get_statistic,rqs)
S3method(get_statistic,rqss)
S3method(get_statistic,scam)
S3method(get_statistic,sdmTMB)
S3method(get_statistic,selection)
S3method(get_statistic,sem)
S3method(get_statistic,summary.gam)
S3method(get_statistic,summary.lm)
S3method(get_statistic,survreg)
S3method(get_statistic,svy2lme)
S3method(get_statistic,svy_vglm)
S3method(get_statistic,svyglm)
S3method(get_statistic,svyglm.nb)
S3method(get_statistic,svyglm.zip)
S3method(get_statistic,svyolr)
S3method(get_statistic,systemfit)
S3method(get_statistic,tobit)
S3method(get_statistic,truncreg)
S3method(get_statistic,vgam)
S3method(get_statistic,vglm)
S3method(get_statistic,wbgee)
S3method(get_statistic,wbm)
S3method(get_statistic,zcpglm)
S3method(get_statistic,zerocount)
S3method(get_statistic,zeroinfl)
S3method(get_varcov,BBmm)
S3method(get_varcov,BBreg)
S3method(get_varcov,DirichletRegModel)
S3method(get_varcov,HLfit)
S3method(get_varcov,LORgee)
S3method(get_varcov,MixMod)
S3method(get_varcov,Rchoice)
S3method(get_varcov,afex_aov)
S3method(get_varcov,aov)
S3method(get_varcov,asym)
S3method(get_varcov,averaging)
S3method(get_varcov,betamfx)
S3method(get_varcov,betaor)
S3method(get_varcov,betareg)
S3method(get_varcov,bife)
S3method(get_varcov,brmsfit)
S3method(get_varcov,cglm)
S3method(get_varcov,clm2)
S3method(get_varcov,clmm2)
S3method(get_varcov,coxr)
S3method(get_varcov,cpglm)
S3method(get_varcov,cpglmm)
S3method(get_varcov,crq)
S3method(get_varcov,crr)
S3method(get_varcov,default)
S3method(get_varcov,estimate_contrasts)
S3method(get_varcov,estimate_means)
S3method(get_varcov,estimate_slopes)
S3method(get_varcov,externVar)
S3method(get_varcov,externX)
S3method(get_varcov,feis)
S3method(get_varcov,fixest)
S3method(get_varcov,flac)
S3method(get_varcov,flexsurvreg)
S3method(get_varcov,flic)
S3method(get_varcov,gamlss)
S3method(get_varcov,gamm)
S3method(get_varcov,gee)
S3method(get_varcov,geeglm)
S3method(get_varcov,glimML)
S3method(get_varcov,glmRob)
S3method(get_varcov,glmgee)
S3method(get_varcov,glmm)
S3method(get_varcov,glmmTMB)
S3method(get_varcov,glmx)
S3method(get_varcov,hglm)
S3method(get_varcov,hurdle)
S3method(get_varcov,ivFixed)
S3method(get_varcov,lcmm)
S3method(get_varcov,list)
S3method(get_varcov,lmRob)
S3method(get_varcov,logitmfx)
S3method(get_varcov,logitor)
S3method(get_varcov,lqm)
S3method(get_varcov,lqmm)
S3method(get_varcov,maxLik)
S3method(get_varcov,mediate)
S3method(get_varcov,merModList)
S3method(get_varcov,mhurdle)
S3method(get_varcov,mixed)
S3method(get_varcov,mixor)
S3method(get_varcov,mjoint)
S3method(get_varcov,mle)
S3method(get_varcov,mle2)
S3method(get_varcov,mlm)
S3method(get_varcov,model_fit)
S3method(get_varcov,mvord)
S3method(get_varcov,negbinirr)
S3method(get_varcov,negbinmfx)
S3method(get_varcov,nestedLogit)
S3method(get_varcov,nlrq)
S3method(get_varcov,pgmm)
S3method(get_varcov,poissonirr)
S3method(get_varcov,poissonmfx)
S3method(get_varcov,probitmfx)
S3method(get_varcov,robmixglm)
S3method(get_varcov,rq)
S3method(get_varcov,rqs)
S3method(get_varcov,sdmTMB)
S3method(get_varcov,selection)
S3method(get_varcov,tobit)
S3method(get_varcov,truncreg)
S3method(get_varcov,vgam)
S3method(get_varcov,vglm)
S3method(get_varcov,zcpglm)
S3method(get_varcov,zerocount)
S3method(get_varcov,zeroinfl)
S3method(get_variance,MixMod)
S3method(get_variance,brmsfit)
S3method(get_variance,clmm)
S3method(get_variance,coxme)
S3method(get_variance,cpglmm)
S3method(get_variance,default)
S3method(get_variance,glmmTMB)
S3method(get_variance,glmmadmb)
S3method(get_variance,lme)
S3method(get_variance,merMod)
S3method(get_variance,mixed)
S3method(get_variance,mjoint)
S3method(get_variance,rlmerMod)
S3method(get_variance,stanreg)
S3method(get_variance,wblm)
S3method(get_variance,wbm)
S3method(get_weights,brmsfit)
S3method(get_weights,btergm)
S3method(get_weights,default)
S3method(get_weights,list)
S3method(get_weights,survey.design)
S3method(get_weights,survey.design2)
S3method(get_weights,svycoxph)
S3method(get_weights,svyglm)
S3method(get_weights,svyolr)
S3method(get_weights,svysurvreg)
S3method(has_intercept,default)
S3method(has_intercept,felm)
S3method(has_intercept,fepois)
S3method(has_intercept,fixest)
S3method(is_converged,"_glm")
S3method(is_converged,default)
S3method(is_converged,glm)
S3method(is_converged,glmmTMB)
S3method(is_converged,lavaan)
S3method(is_converged,merMod)
S3method(is_mixed_model,afex_aov)
S3method(is_mixed_model,default)
S3method(is_mixed_model,marginaleffects)
S3method(is_mixed_model,wbgee)
S3method(is_mixed_model,wbm)
S3method(is_nullmodel,afex_aov)
S3method(is_nullmodel,default)
S3method(link_function,BBmm)
S3method(link_function,BBreg)
S3method(link_function,DirichletRegModel)
S3method(link_function,LORgee)
S3method(link_function,MANOVA)
S3method(link_function,MCMCglmm)
S3method(link_function,RM)
S3method(link_function,Rchoice)
S3method(link_function,afex_aov)
S3method(link_function,aovlist)
S3method(link_function,asym)
S3method(link_function,averaging)
S3method(link_function,bamlss)
S3method(link_function,bayesx)
S3method(link_function,bcplm)
S3method(link_function,betamfx)
S3method(link_function,betaor)
S3method(link_function,betareg)
S3method(link_function,bife)
S3method(link_function,bigglm)
S3method(link_function,biglm)
S3method(link_function,brglm)
S3method(link_function,brmsfit)
S3method(link_function,censReg)
S3method(link_function,cgam)
S3method(link_function,cglm)
S3method(link_function,clm)
S3method(link_function,clm2)
S3method(link_function,clmm)
S3method(link_function,complmRob)
S3method(link_function,comprisk)
S3method(link_function,coxme)
S3method(link_function,coxph)
S3method(link_function,coxr)
S3method(link_function,cpglm)
S3method(link_function,cpglmm)
S3method(link_function,cph)
S3method(link_function,crch)
S3method(link_function,crq)
S3method(link_function,crqs)
S3method(link_function,default)
S3method(link_function,externVar)
S3method(link_function,externX)
S3method(link_function,feglm)
S3method(link_function,feis)
S3method(link_function,felm)
S3method(link_function,fixest)
S3method(link_function,flac)
S3method(link_function,flexsurvreg)
S3method(link_function,flic)
S3method(link_function,gam)
S3method(link_function,gamlss)
S3method(link_function,gamm)
S3method(link_function,gbm)
S3method(link_function,glimML)
S3method(link_function,glm)
S3method(link_function,glmgee)
S3method(link_function,glmm)
S3method(link_function,glmmadmb)
S3method(link_function,glmx)
S3method(link_function,gls)
S3method(link_function,gmnl)
S3method(link_function,hurdle)
S3method(link_function,ivFixed)
S3method(link_function,iv_robust)
S3method(link_function,ivprobit)
S3method(link_function,ivreg)
S3method(link_function,lm)
S3method(link_function,lmRob)
S3method(link_function,lm_robust)
S3method(link_function,lme)
S3method(link_function,lmrob)
S3method(link_function,logistf)
S3method(link_function,logitmfx)
S3method(link_function,logitor)
S3method(link_function,logitr)
S3method(link_function,lqm)
S3method(link_function,lqmm)
S3method(link_function,lrm)
S3method(link_function,mblogit)
S3method(link_function,mclogit)
S3method(link_function,merModList)
S3method(link_function,mipo)
S3method(link_function,mira)
S3method(link_function,mixed)
S3method(link_function,mixor)
S3method(link_function,mlogit)
S3method(link_function,mmclogit)
S3method(link_function,mmrm)
S3method(link_function,mmrm_fit)
S3method(link_function,mmrm_tmb)
S3method(link_function,model_fit)
S3method(link_function,multinom)
S3method(link_function,multinom_weightit)
S3method(link_function,mvord)
S3method(link_function,negbinirr)
S3method(link_function,negbinmfx)
S3method(link_function,nestedLogit)
S3method(link_function,oohbchoice)
S3method(link_function,ordinal_weightit)
S3method(link_function,orm)
S3method(link_function,phyloglm)
S3method(link_function,phylolm)
S3method(link_function,plm)
S3method(link_function,poissonirr)
S3method(link_function,poissonmfx)
S3method(link_function,polr)
S3method(link_function,probitmfx)
S3method(link_function,psm)
S3method(link_function,riskRegression)
S3method(link_function,robmixglm)
S3method(link_function,rq)
S3method(link_function,rqs)
S3method(link_function,rqss)
S3method(link_function,sdmTMB)
S3method(link_function,serp)
S3method(link_function,speedglm)
S3method(link_function,speedlm)
S3method(link_function,stanmvreg)
S3method(link_function,survfit)
S3method(link_function,survreg)
S3method(link_function,svy2lme)
S3method(link_function,svy_vglm)
S3method(link_function,svyolr)
S3method(link_function,systemfit)
S3method(link_function,tobit)
S3method(link_function,truncreg)
S3method(link_function,vgam)
S3method(link_function,vglm)
S3method(link_function,zcpglm)
S3method(link_function,zeroinfl)
S3method(link_function,zerotrunc)
S3method(link_inverse,BBmm)
S3method(link_inverse,BBreg)
S3method(link_inverse,DirichletRegModel)
S3method(link_inverse,LORgee)
S3method(link_inverse,MANOVA)
S3method(link_inverse,MCMCglmm)
S3method(link_inverse,MixMod)
S3method(link_inverse,RM)
S3method(link_inverse,Rchoice)
S3method(link_inverse,afex_aov)
S3method(link_inverse,aovlist)
S3method(link_inverse,asym)
S3method(link_inverse,averaging)
S3method(link_inverse,bamlss)
S3method(link_inverse,bayesx)
S3method(link_inverse,bcplm)
S3method(link_inverse,betamfx)
S3method(link_inverse,betaor)
S3method(link_inverse,betareg)
S3method(link_inverse,bife)
S3method(link_inverse,bigglm)
S3method(link_inverse,biglm)
S3method(link_inverse,brmsfit)
S3method(link_inverse,censReg)
S3method(link_inverse,cgam)
S3method(link_inverse,cglm)
S3method(link_inverse,clm)
S3method(link_inverse,clm2)
S3method(link_inverse,clmm)
S3method(link_inverse,complmrob)
S3method(link_inverse,comprisk)
S3method(link_inverse,coxme)
S3method(link_inverse,coxph)
S3method(link_inverse,coxr)
S3method(link_inverse,cpglm)
S3method(link_inverse,cpglmm)
S3method(link_inverse,cph)
S3method(link_inverse,crch)
S3method(link_inverse,crq)
S3method(link_inverse,crqs)
S3method(link_inverse,default)
S3method(link_inverse,externVar)
S3method(link_inverse,externX)
S3method(link_inverse,feglm)
S3method(link_inverse,feis)
S3method(link_inverse,felm)
S3method(link_inverse,fixest)
S3method(link_inverse,flac)
S3method(link_inverse,flexsurvreg)
S3method(link_inverse,flic)
S3method(link_inverse,gam)
S3method(link_inverse,gamlss)
S3method(link_inverse,gamm)
S3method(link_inverse,gbm)
S3method(link_inverse,glimML)
S3method(link_inverse,glm)
S3method(link_inverse,glmgee)
S3method(link_inverse,glmm)
S3method(link_inverse,glmmPQL)
S3method(link_inverse,glmmTMB)
S3method(link_inverse,glmmadmb)
S3method(link_inverse,glmx)
S3method(link_inverse,gls)
S3method(link_inverse,gmnl)
S3method(link_inverse,hurdle)
S3method(link_inverse,ivFixed)
S3method(link_inverse,iv_robust)
S3method(link_inverse,ivprobit)
S3method(link_inverse,ivreg)
S3method(link_inverse,lm)
S3method(link_inverse,lmRob)
S3method(link_inverse,lm_robust)
S3method(link_inverse,lme)
S3method(link_inverse,lmrob)
S3method(link_inverse,logistf)
S3method(link_inverse,logitmfx)
S3method(link_inverse,logitor)
S3method(link_inverse,logitr)
S3method(link_inverse,lqm)
S3method(link_inverse,lqmm)
S3method(link_inverse,lrm)
S3method(link_inverse,mblogit)
S3method(link_inverse,mclogit)
S3method(link_inverse,merModList)
S3method(link_inverse,mipo)
S3method(link_inverse,mira)
S3method(link_inverse,mixed)
S3method(link_inverse,mixor)
S3method(link_inverse,mlogit)
S3method(link_inverse,mmclogit)
S3method(link_inverse,mmrm)
S3method(link_inverse,mmrm_fit)
S3method(link_inverse,mmrm_tmb)
S3method(link_inverse,model_fit)
S3method(link_inverse,multinom)
S3method(link_inverse,multinom_weightit)
S3method(link_inverse,mvord)
S3method(link_inverse,negbinirr)
S3method(link_inverse,negbinmfx)
S3method(link_inverse,nestedLogit)
S3method(link_inverse,oohbchoice)
S3method(link_inverse,ordinal_weightit)
S3method(link_inverse,orm)
S3method(link_inverse,phyloglm)
S3method(link_inverse,phylolm)
S3method(link_inverse,plm)
S3method(link_inverse,poissonmfx)
S3method(link_inverse,polr)
S3method(link_inverse,probitirr)
S3method(link_inverse,probitmfx)
S3method(link_inverse,psm)
S3method(link_inverse,riskRegression)
S3method(link_inverse,robmixglm)
S3method(link_inverse,rq)
S3method(link_inverse,rqs)
S3method(link_inverse,rqss)
S3method(link_inverse,sdmTMB)
S3method(link_inverse,serp)
S3method(link_inverse,speedglm)
S3method(link_inverse,speedlm)
S3method(link_inverse,stanmvreg)
S3method(link_inverse,survfit)
S3method(link_inverse,survreg)
S3method(link_inverse,svy2lme)
S3method(link_inverse,svy_vglm)
S3method(link_inverse,svyolr)
S3method(link_inverse,systemfit)
S3method(link_inverse,tobit)
S3method(link_inverse,truncreg)
S3method(link_inverse,vgam)
S3method(link_inverse,vglm)
S3method(link_inverse,zcpglm)
S3method(link_inverse,zeroinfl)
S3method(link_inverse,zerotrunc)
S3method(model_info,Arima)
S3method(model_info,BBmm)
S3method(model_info,BBreg)
S3method(model_info,BFBayesFactor)
S3method(model_info,BGGM)
S3method(model_info,DirichletRegModel)
S3method(model_info,LORgee)
S3method(model_info,MANOVA)
S3method(model_info,MCMCglmm)
S3method(model_info,MixMod)
S3method(model_info,RM)
S3method(model_info,Rchoice)
S3method(model_info,aareg)
S3method(model_info,afex_aov)
S3method(model_info,aft)
S3method(model_info,anova)
S3method(model_info,aovlist)
S3method(model_info,asym)
S3method(model_info,averaging)
S3method(model_info,bamlss)
S3method(model_info,bayesQR)
S3method(model_info,bayesx)
S3method(model_info,bcplm)
S3method(model_info,betamfx)
S3method(model_info,betaor)
S3method(model_info,betareg)
S3method(model_info,bfsl)
S3method(model_info,bife)
S3method(model_info,blrm)
S3method(model_info,brmsfit)
S3method(model_info,brmultinom)
S3method(model_info,censReg)
S3method(model_info,cgam)
S3method(model_info,cglm)
S3method(model_info,clm)
S3method(model_info,clm2)
S3method(model_info,clmm)
S3method(model_info,coeftest)
S3method(model_info,complmrob)
S3method(model_info,comprisk)
S3method(model_info,coxme)
S3method(model_info,coxph)
S3method(model_info,coxr)
S3method(model_info,cpglm)
S3method(model_info,cpglmm)
S3method(model_info,crch)
S3method(model_info,crq)
S3method(model_info,crqs)
S3method(model_info,data.frame)
S3method(model_info,default)
S3method(model_info,deltaMethod)
S3method(model_info,earth)
S3method(model_info,externVar)
S3method(model_info,externX)
S3method(model_info,feglm)
S3method(model_info,feis)
S3method(model_info,felm)
S3method(model_info,fixest)
S3method(model_info,flac)
S3method(model_info,flexsurvreg)
S3method(model_info,flic)
S3method(model_info,gam)
S3method(model_info,gamlss)
S3method(model_info,gamm)
S3method(model_info,garch)
S3method(model_info,gbm)
S3method(model_info,ggcomparisons)
S3method(model_info,glht)
S3method(model_info,glimML)
S3method(model_info,glmgee)
S3method(model_info,glmm)
S3method(model_info,glmmPQL)
S3method(model_info,glmmTMB)
S3method(model_info,glmmadmb)
S3method(model_info,glmx)
S3method(model_info,gls)
S3method(model_info,gmnl)
S3method(model_info,hglm)
S3method(model_info,htest)
S3method(model_info,hurdle)
S3method(model_info,ivFixed)
S3method(model_info,iv_robust)
S3method(model_info,ivprobit)
S3method(model_info,ivreg)
S3method(model_info,lcmm)
S3method(model_info,lmRob)
S3method(model_info,lm_robust)
S3method(model_info,lme)
S3method(model_info,lmrob)
S3method(model_info,logistf)
S3method(model_info,logitmfx)
S3method(model_info,logitor)
S3method(model_info,logitr)
S3method(model_info,lqm)
S3method(model_info,lqmm)
S3method(model_info,lrm)
S3method(model_info,marginaleffects)
S3method(model_info,maxLik)
S3method(model_info,mblogit)
S3method(model_info,mclogit)
S3method(model_info,mcmc)
S3method(model_info,merModList)
S3method(model_info,meta_bma)
S3method(model_info,meta_fixed)
S3method(model_info,meta_random)
S3method(model_info,metaplus)
S3method(model_info,mhurdle)
S3method(model_info,mipo)
S3method(model_info,mira)
S3method(model_info,mixed)
S3method(model_info,mixor)
S3method(model_info,mjoint)
S3method(model_info,mlm)
S3method(model_info,mlogit)
S3method(model_info,mmclogit)
S3method(model_info,mmrm)
S3method(model_info,mmrm_fit)
S3method(model_info,mmrm_tmb)
S3method(model_info,model_fit)
S3method(model_info,multinom)
S3method(model_info,multinom_weightit)
S3method(model_info,mvord)
S3method(model_info,negbinirr)
S3method(model_info,negbinmfx)
S3method(model_info,nestedLogit)
S3method(model_info,nlrq)
S3method(model_info,nls)
S3method(model_info,oohbchoice)
S3method(model_info,ordinal_weightit)
S3method(model_info,orm)
S3method(model_info,phyloglm)
S3method(model_info,phylolm)
S3method(model_info,plm)
S3method(model_info,poissonirr)
S3method(model_info,poissonmfx)
S3method(model_info,polr)
S3method(model_info,probitmfx)
S3method(model_info,pstpm2)
S3method(model_info,riskRegression)
S3method(model_info,rma)
S3method(model_info,robmixglm)
S3method(model_info,rq)
S3method(model_info,rqs)
S3method(model_info,rqss)
S3method(model_info,sdmTMB)
S3method(model_info,serp)
S3method(model_info,speedglm)
S3method(model_info,speedlm)
S3method(model_info,stanmvreg)
S3method(model_info,stanreg)
S3method(model_info,stpm2)
S3method(model_info,summary.lm)
S3method(model_info,survfit)
S3method(model_info,survreg)
S3method(model_info,svy2lme)
S3method(model_info,svy_vglm)
S3method(model_info,svyolr)
S3method(model_info,systemfit)
S3method(model_info,tobit)
S3method(model_info,truncreg)
S3method(model_info,vgam)
S3method(model_info,vglm)
S3method(model_info,zcpglm)
S3method(model_info,zeroinfl)
S3method(model_info,zerotrunc)
S3method(model_name,default)
S3method(model_name,list)
S3method(n_obs,BBmm)
S3method(n_obs,BBreg)
S3method(n_obs,Glm)
S3method(n_obs,HLfit)
S3method(n_obs,LORgee)
S3method(n_obs,MANOVA)
S3method(n_obs,RM)
S3method(n_obs,Rchoice)
S3method(n_obs,SemiParBIV)
S3method(n_obs,aareg)
S3method(n_obs,afex_aov)
S3method(n_obs,aovlist)
S3method(n_obs,averaging)
S3method(n_obs,bamlss)
S3method(n_obs,bayesx)
S3method(n_obs,bcplm)
S3method(n_obs,betamfx)
S3method(n_obs,betaor)
S3method(n_obs,bife)
S3method(n_obs,bigglm)
S3method(n_obs,biglm)
S3method(n_obs,blrm)
S3method(n_obs,censReg)
S3method(n_obs,cgam)
S3method(n_obs,cglm)
S3method(n_obs,coeftest)
S3method(n_obs,complmrob)
S3method(n_obs,comprisk)
S3method(n_obs,coxme)
S3method(n_obs,coxph)
S3method(n_obs,coxr)
S3method(n_obs,cpglm)
S3method(n_obs,cpglmm)
S3method(n_obs,crq)
S3method(n_obs,crqs)
S3method(n_obs,crr)
S3method(n_obs,default)
S3method(n_obs,eglm)
S3method(n_obs,emm_list)
S3method(n_obs,externVar)
S3method(n_obs,externX)
S3method(n_obs,feglm)
S3method(n_obs,feis)
S3method(n_obs,felm)
S3method(n_obs,fixest)
S3method(n_obs,flexsurvreg)
S3method(n_obs,gam)
S3method(n_obs,gamm)
S3method(n_obs,garch)
S3method(n_obs,gbm)
S3method(n_obs,glimML)
S3method(n_obs,glm)
S3method(n_obs,glmRob)
S3method(n_obs,glmgee)
S3method(n_obs,gmnl)
S3method(n_obs,hurdle)
S3method(n_obs,ivFixed)
S3method(n_obs,ivprobit)
S3method(n_obs,joint)
S3method(n_obs,lavaan)
S3method(n_obs,lcmm)
S3method(n_obs,list)
S3method(n_obs,lmRob)
S3method(n_obs,lmodel2)
S3method(n_obs,logitmfx)
S3method(n_obs,logitor)
S3method(n_obs,logitr)
S3method(n_obs,lqm)
S3method(n_obs,lqmm)
S3method(n_obs,marginaleffects)
S3method(n_obs,maxLik)
S3method(n_obs,mcmc)
S3method(n_obs,mediate)
S3method(n_obs,merModList)
S3method(n_obs,mhurdle)
S3method(n_obs,mipo)
S3method(n_obs,mira)
S3method(n_obs,mjoint)
S3method(n_obs,mle)
S3method(n_obs,mle2)
S3method(n_obs,mlogit)
S3method(n_obs,model_fit)
S3method(n_obs,multinom)
S3method(n_obs,mvord)
S3method(n_obs,negbinirr)
S3method(n_obs,negbinmfx)
S3method(n_obs,nestedLogit)
S3method(n_obs,nlrq)
S3method(n_obs,phyloglm)
S3method(n_obs,phylolm)
S3method(n_obs,poissonirr)
S3method(n_obs,poissonmfx)
S3method(n_obs,probitmfx)
S3method(n_obs,riskRegression)
S3method(n_obs,rq)
S3method(n_obs,rqs)
S3method(n_obs,rqss)
S3method(n_obs,selection)
S3method(n_obs,sem)
S3method(n_obs,stanmvreg)
S3method(n_obs,summary.lm)
S3method(n_obs,survfit)
S3method(n_obs,survreg)
S3method(n_obs,svy_vglm)
S3method(n_obs,svyolr)
S3method(n_obs,wbgee)
S3method(n_obs,wbm)
S3method(n_obs,zcpglm)
S3method(n_obs,zeroinfl)
S3method(n_obs,zerotrunc)
S3method(n_parameters,BBmm)
S3method(n_parameters,Gam)
S3method(n_parameters,MCMCglmm)
S3method(n_parameters,MixMod)
S3method(n_parameters,bayesx)
S3method(n_parameters,blavaan)
S3method(n_parameters,brmsfit)
S3method(n_parameters,coxme)
S3method(n_parameters,cpglmm)
S3method(n_parameters,default)
S3method(n_parameters,gam)
S3method(n_parameters,glimML)
S3method(n_parameters,glmmTMB)
S3method(n_parameters,gls)
S3method(n_parameters,hurdle)
S3method(n_parameters,lavaan)
S3method(n_parameters,lme)
S3method(n_parameters,logitr)
S3method(n_parameters,merMod)
S3method(n_parameters,mixed)
S3method(n_parameters,multinom)
S3method(n_parameters,rlmerMod)
S3method(n_parameters,sim.merMod)
S3method(n_parameters,stanmvreg)
S3method(n_parameters,stanreg)
S3method(n_parameters,svy2lme)
S3method(n_parameters,vgam)
S3method(n_parameters,wbm)
S3method(n_parameters,zeroinfl)
S3method(n_parameters,zerotrunc)
S3method(n_unique,data.frame)
S3method(n_unique,default)
S3method(n_unique,list)
S3method(null_model,MixMod)
S3method(null_model,clm2)
S3method(null_model,coxme)
S3method(null_model,cpglmm)
S3method(null_model,default)
S3method(null_model,glmerMod)
S3method(null_model,glmmTMB)
S3method(null_model,glmmadmb)
S3method(null_model,lmerMod)
S3method(null_model,merMod)
S3method(null_model,mixed)
S3method(null_model,multinom)
S3method(null_model,nlmerMod)
S3method(null_model,rlmerMod)
S3method(print,check_if_installed)
S3method(print,easystats_check)
S3method(print,get_predicted)
S3method(print,insight_residuals)
S3method(print,insight_table)
S3method(print,n_grouplevels)
S3method(print_html,array)
S3method(print_html,data.frame)
S3method(print_html,matrix)
S3method(print_html,table)
S3method(print_md,array)
S3method(print_md,data.frame)
S3method(print_md,matrix)
S3method(print_md,table)
S3method(standardize_column_order,data.frame)
S3method(standardize_column_order,default)
S3method(standardize_column_order,effectsize_table)
S3method(standardize_column_order,parameters_distribution)
S3method(standardize_column_order,parameters_model)
S3method(standardize_names,data.frame)
S3method(standardize_names,default)
S3method(standardize_names,effectsize_table)
S3method(standardize_names,parameters_distribution)
S3method(standardize_names,parameters_model)
S3method(summary,get_predicted)
S3method(text_remove_backticks,character)
S3method(text_remove_backticks,data.frame)
S3method(text_remove_backticks,default)
S3method(text_remove_backticks,factor)
S3method(text_remove_backticks,list)
S3method(text_remove_backticks,matrix)
S3method(trim_ws,data.frame)
S3method(trim_ws,default)
S3method(trim_ws,list)
S3method(vcovFPC,clm)
S3method(vcovFPC,clm2)
S3method(vcovFPC,default)
S3method(vcovFPC,glm)
S3method(vcovFPC,glmRob)
S3method(vcovFPC,glmmTMB)
S3method(vcovFPC,glmrob)
S3method(vcovFPC,lm)
S3method(vcovFPC,lmRob)
S3method(vcovFPC,lm_robust)
S3method(vcovFPC,merMod)
S3method(vcovFPC,model_fit)
S3method(vcovFPC,polr)
S3method(vcovFPC,speedglm)
S3method(vcovFPC,speedlm)
export(all_models_equal)
export(all_models_same_class)
export(broom_columns)
export(check_if_installed)
export(clean_names)
export(clean_parameters)
export(color_if)
export(color_text)
export(color_theme)
export(colour_if)
export(colour_text)
export(compact_character)
export(compact_list)
export(display)
export(download_model)
export(easystats_columns)
export(ellipsis_info)
export(export_table)
export(find_algorithm)
export(find_auxiliary)
export(find_formula)
export(find_interactions)
export(find_offset)
export(find_parameters)
export(find_predictors)
export(find_random)
export(find_random_slopes)
export(find_response)
export(find_smooth)
export(find_statistic)
export(find_terms)
export(find_transformation)
export(find_variables)
export(find_weights)
export(format_alert)
export(format_bf)
export(format_capitalize)
export(format_ci)
export(format_error)
export(format_message)
export(format_number)
export(format_p)
export(format_pd)
export(format_percent)
export(format_rope)
export(format_string)
export(format_table)
export(format_value)
export(format_warning)
export(formula_ok)
export(get_auxiliary)
export(get_call)
export(get_correlation_slope_intercept)
export(get_correlation_slopes)
export(get_data)
export(get_datagrid)
export(get_deviance)
export(get_df)
export(get_dispersion)
export(get_family)
export(get_intercept)
export(get_loglikelihood)
export(get_loglikelihood_adjustment)
export(get_mixed_info)
export(get_model)
export(get_modelmatrix)
export(get_parameters)
export(get_predicted)
export(get_predicted_ci)
export(get_predictors)
export(get_priors)
export(get_random)
export(get_residuals)
export(get_response)
export(get_sigma)
export(get_simulated)
export(get_statistic)
export(get_transformation)
export(get_varcov)
export(get_variance)
export(get_variance_dispersion)
export(get_variance_distribution)
export(get_variance_fixed)
export(get_variance_intercept)
export(get_variance_random)
export(get_variance_residual)
export(get_variance_slope)
export(get_weights)
export(has_intercept)
export(has_single_value)
export(is_bayesian_model)
export(is_converged)
export(is_empty_object)
export(is_gam_model)
export(is_mixed_model)
export(is_model)
export(is_model_supported)
export(is_multivariate)
export(is_nested_models)
export(is_nullmodel)
export(is_regression_model)
export(link_function)
export(link_inverse)
export(loglikelihood)
export(model_info)
export(model_name)
export(n_grouplevels)
export(n_obs)
export(n_parameters)
export(n_unique)
export(null_model)
export(object_has_names)
export(object_has_rownames)
export(print_color)
export(print_colour)
export(print_html)
export(print_md)
export(print_parameters)
export(safe_deparse)
export(safe_deparse_symbol)
export(standardize_column_order)
export(standardize_names)
export(supported_models)
export(text_remove_backticks)
export(trim_ws)
export(validate_argument)
export(vcovFPC)
