if (requiet("testthat") && requiet("insight")) {
  test_that("export_table", {
    # mimic package name if cat were to walk on a keyboard
    expect_error(check_if_installed("xklfueofi8eur3rnfalfb"))
  })
}
