# nolint start
# styler: off
structure(
  list(
    method = "GET",
    url = "https://raw.githubusercontent.com/easystats/circus/master/data/xyz.rda",
    status_code = 404L,
    headers = structure(
      list(
        `content-security-policy` = "default-src 'none'; style-src 'unsafe-inline'; sandbox",
        `strict-transport-security` = "max-age=31536000",
        `x-content-type-options` = "nosniff",
        `x-frame-options` = "deny",
        `x-xss-protection` = "1; mode=block",
        `content-type` = "text/plain; charset=utf-8",
        `x-github-request-id` = "63EC:2A5B9F:A27173:ADAE50:66655C62",
        `accept-ranges` = "bytes",
        date = "Sun, 09 Jun 2024 07:40:19 GMT",
        via = "1.1 varnish",
        `x-served-by` = "cache-muc13972-MUC",
        `x-cache` = "MISS",
        `x-cache-hits` = "0",
        `x-timer` = "S1717918819.042740,VS0,VE145",
        vary = "Authorization,Accept-Encoding,Origin",
        `access-control-allow-origin` = "*",
        `cross-origin-resource-policy` = "cross-origin",
        `x-fastly-request-id` = "e629ba1c490defdad14a749303c5f1649289f72c",
        expires = "Sun, 09 Jun 2024 07:45:19 GMT",
        `source-age` = "0",
        `content-length` = "14"
      ),
      class = "httr2_headers"
    ),
    body = charToRaw("404: Not Found"),
    cache = new.env(parent = emptyenv())
  ),
  class = "httr2_response"
)
# styler: on
# nolint end
