% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_if.R
\name{.colour_detect}
\alias{.colour_detect}
\title{Detect coloured cells}
\usage{
.colour_detect(x)
}
\description{
Detect coloured cells
}
\keyword{internal}
